<?php
/**
 * Copyright (c) 2020 TechDivision GmbH
 * All rights reserved
 *
 * This product includes proprietary software developed at TechDivision GmbH, Germany
 * For more information see http://www.techdivision.com/
 *
 * To obtain a valid license for using this software please contact us at
 * license@techdivision.com
 */
declare(strict_types=1);

namespace TechDivision\OrderWorkflowCustomExportFormat\Model;

use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Filesystem\Io\File;
use Magento\Sales\Api\OrderRepositoryInterface;
use TechDivision\PacemakerImportBase\Model\AbsolutePathResolver;
use TechDivision\ProcessPipelines\Api\ExecutorLoggerInterface;
use TechDivision\ProcessPipelines\Api\StepInterface;
use TechDivision\ProcessPipelines\Model\Executor\AbstractExecutor;

/**
 * @copyright  Copyright (c) 2020 TechDivision GmbH (http://www.techdivision.com)
 * @link       http://www.techdivision.com/
 * @author     Vadim Justus <v.justus@techdivision.com>
 */
class CustomOrderFormatExecutor extends AbstractExecutor
{
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var File
     */
    private $file;

    /**
     * @var AbsolutePathResolver
     */
    private $absolutePathResolver;

    /**
     * @var MyFormatRenderer
     */
    private $myFormatRenderer;

    /**
     * @param ExecutorLoggerInterface $logger
     * @param OrderRepositoryInterface $orderRepository
     * @param File $file
     * @param AbsolutePathResolver $absolutePathResolver
     * @param MyFormatRenderer $myFormatRenderer
     */
    public function __construct(
        ExecutorLoggerInterface $logger,
        OrderRepositoryInterface $orderRepository,
        File $file,
        AbsolutePathResolver $absolutePathResolver,
        MyFormatRenderer $myFormatRenderer
    ) {
        parent::__construct($logger);
        $this->orderRepository = $orderRepository;
        $this->file = $file;
        $this->absolutePathResolver = $absolutePathResolver;
        $this->myFormatRenderer = $myFormatRenderer;
    }

    /**
     * @param StepInterface $step
     * @return void
     * @throws LocalizedException
     */
    public function process(StepInterface $step)
    {
        // Load the order with the order_id given in step arguments
        $order = $this->orderRepository->get((int)$step->getArgumentValueByKey('order_id'));

        // It's recommended to use logger to have better monitoring
        $this->getLogger()->info(
            sprintf(
                'Order %s (%s) was loaded!',
                $order->getIncrementId(),
                $order->getEntityId()
            )
        );

        // Generate the target order format
        $result = $this->myFormatRenderer->execute($order, $this->getLogger());

        // retrieve absolute working directory path
        $workingDirectory = $this->absolutePathResolver->get($step->getWorkingDirectory());

        // define the file path to persist your results
        // the default local file system transporter will check for
        // the file order-export.body within the working dir
        $bodyFile = $workingDirectory . '/order-export.body';
        $this->file->write($bodyFile, $result);

        // Do not forget to log the process ;)
        $this->getLogger()->info(sprintf('Persist transformed order in file "%s"', $bodyFile));
    }
}
